#!/usr/local/bin/perl

#------------------------------------------------------------------------------
# ecescan.pm
#
# perl module used in the ecescan project with basic data conversion and
# database handling abilities
#
# chok, last updated on tue 9 jul 2002
#------------------------------------------------------------------------------

package ecescan;
require Exporter;
@ISA    = qw(Exporter);
@EXPORT = qw(
    long_to_ip
    ip_to_long
    get_count_data
    set_count_data
    increment_count_data
    connectdatabase
    enddatabase
    SQL_do
    SQL_prepare_execute
    SQL_prepare_execute1
    SQL_fetchrow
    SQL_fetchrow1
    SQL_fetchrow_hashref
    SQL_finishstatement
    SQL_finishstatement1
    readable_timestamp
    get_time_seg
    sum_count
);

use Socket;
use DBI;


#--- read from data file ----------------------------------
do( './ecescan.data' );

1;			# req'd to prevent error in "use ecescan"


#------------------------------------------------------------------------------
# long_to_ip()
#
# convert a network long int to an IP address
#
# $lip - ip addr in long int (4 bytes) format, number
#
# retval: ip addr in xxx.xxx.xxx.xxx format, string
#
sub long_to_ip # $lip
{
    return inet_ntoa( pack( 'N', $_[0] ) );
}
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
# ip_to_long()
#
# convert an IP address to a long int
#
# $ip - ip addr in xxx.xxx.xxx.xxx format, string
#
# retval: ip addr in network long int (4 bytes) format, number
#
sub ip_to_long # $ip
{
    return unpack( 'N', inet_aton( $_[0] ) );
}
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
# get_count_data()
#
# retrieve a single number found in the count data
#
# \$str - string containing the numbers
# $seg - time segment no of the data to retrieve
#
# retval: the number specified, string of $size_element chars or less
#
sub get_count_data # \$str, $seg
{
    $ptr = $_[0]; $seg = $_[ 1 ];

    $offset = $seg * $size_element;
    if( length( $$ptr ) <= $offset )
        { return '0'; }   # beyond defined range, so value = 0
    else
        { return ( substr( $$ptr, $offset, $size_element ) + 0 ); }
            # '+ 0' behind converts string to number
}
#-------------------------------------------------------------------------------


#------------------------------------------------------------------------------
# set_count_data()
#
# set the value of a single number in the count data
#
# \$str - string containing the numbers
# $seg - time segment no of the data to set
# $val - the value to set the data to
#
# retval: $str is modified with data item set to $val
#
sub set_count_data # \$str, $seg, $val
{
    $ptr = $_[0]; $seg = $_[ 1 ]; $val = $_[ 2 ];

    # extend the size of the given string
    $$ptr .= $null_fill_char x ( $size_data - length( $$ptr ) );

    # process the substring
    $s = $val . ( $null_fill_char x ( $size_element - length( $val ) ) );

    # re-insert the substring
    substr( $$ptr, $seg * $size_element, $size_element ) = $s;
}
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
# increment_count_data()
#
# increment the value of a single number in the count data
#
# \$str - pointer to string containing the numbers
# $seg - time segment no of the data to increment by one
#
# retval: $str is modified with selected data incremented
#
sub increment_count_data # \$str, $seg
{
    $ptr = $_[0]; $seg = $_[ 1 ];

    # extend the size of the given string
    $$ptr .= $null_fill_char x ( $size_data - length( $$ptr ) );

    # extract the sub-string
    $offset = $seg * $size_element;
    $s = substr( $$ptr, $offset, $size_element );

    # process the substring
    $s ++; $s .= ( $null_fill_char x ( $size_element - length( $s ) ) );

    # re-insert the substring
    substr( $$ptr, $offset, $size_element ) = $s;
}
#------------------------------------------------------------------------------


sub connectdatabase # no args
{
    $dbh = DBI->connect( $db_name, $db_user, $db_pw,
       {
           RaiseError => 1,
           AutoCommit => 1
       } );

   # RaiseError = 1 => the "or die errmsg" task will be performed automatically
   #                   for each call to a database method, therefore there is
   #                   no need to specify "or die" in any DBI cmds
   # AutoCommit = 1 => changes to database will be written immediately, so that
   #                   there is no need to use the "$dbh->commit()" method
}

sub enddatabase # no args
{
	$dbh->disconnect();
}

sub SQL_do # args: $sql
{
    $dbh->do($_[0]) or print("SQL_do($_[0])");
}

sub SQL_prepare_execute # $sql
{
    $sth = $dbh->prepare( $_[0] );
    return $sth->execute();
}

sub SQL_prepare_execute1 # $sql
	{ $sth1 = $dbh->prepare( $_[0] ); return $sth1->execute(); }

sub SQL_fetchrow # no args
	{return $sth->fetchrow_array();}

sub SQL_fetchrow1 # no args
	{return $sth1->fetchrow_array();}

sub SQL_fetchrow_hashref # no args
	{return $sth->fetchrow_hashref();}

sub SQL_finishstatement # no args
	{$sth->finish();}

sub SQL_finishstatement1 # no args
	{$sth1->finish();}



#------------------------------------------------------------------------------
# readable_timestamp()
#
# returns the timestamp date & time in human readable form
#
# $ts - timestamp extracted from database, in format yyyymmddhhmmss, string
#
# retval: given date & time in human readable form
#
sub readable_timestamp # $ts
{
    return sprintf( "%d%d%d%d-%d%d-%d%d %d%d:%d%d:%d%d", split( '', $_[0] ) );
}
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
# get_time_seg()
#
# returns time segment number to store in database, -1 for out of range
# first interval from time_first <= time < $time_first + $time_interval
# note that if current time = last time (1700 hrs), then the segment
# number would be -1
#
# $hr  - hour of time to calculate for
# $min - minute of time to calculate for
#
# retval: 0..(no_of_time_slots-1), -1 for out of range
#
sub get_time_seg # $hr, $min
{
    $hr = $_[0]; $min = $_[ 1 ];

    # convert time: from 0000-2400 to 0000-1440 (24 * 60)
    $curtime   = $hr * 60 + $min;
    $firsttime = int( $time_first / 100 ) * 60 + ( $time_first % 100 );
    $lasttime  = int( $time_last / 100 ) * 60 + ( $time_last % 100 );

    # determine time segment no
    if( $lasttime <= $firsttime )
        # shift $lasttime by one day if it falls out of range
        { $lasttime += 24 * 60; }
    if( $curtime < $firsttime )
        # shift $curtime by one day if it falls out of range
        { $curtime += 24 * 60; }

    if( $curtime >= $firsttime && $curtime < $lasttime )
        # within time range
        { $time_segment_no = int( ( $curtime - $firsttime ) / $time_interval ); }
    else
        #out of range
        { $time_segment_no = $TIME_SEG_ERROR; }

    return $time_segment_no;
}
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
# sum_count()
#
# returns the sum of all the individual counts as stored in the coded strings
# either under column timeactivecount or timetotalcount, for report-generating
# purposes
#
# \$str - pointer to encoded string of counts extracted straight from database
#
# retval: total sum of all the long int values in the given string
#
sub sum_count # \$str
{
    $ptr = $_[0];   # $$ptr == $cntstr in ecescan
    $sum = 0;

    # any index after the $last_non_zero_index has a value of zero
    $last_non_zero_index = int( length( $$ptr ) / $size_element );

    for $i ( 0 .. $last_non_zero_index )
    {
        # extract the substring in the $$ptr and add it to $sum
        $sum += substr( $$ptr, $i * $size_element, $size_element );
    }
    return $sum;
}
#------------------------------------------------------------------------------

