
/* RCS Information:
 *
 * $Id$
 * $Revision$
 * $Date$
 * $Author$
 * $Locker$
 * 
 */

/*****************************************************************************
 *
 *  Copyright (C) 2002,  Bill Pierce 
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Bill Pierce can be contacted at:
 *  7862 NE 148th St. 
 *  Kenmore, WA 98028
 *
 *****************************************************************************/

/*****************************************************************************
 *
 * fcping
 *
 * Command-line utility that allows the user to try to ping SCSI devices 
 * in-band through a suitably configured HBA in a host. 
 *
 * This program sits on top of the the library hba_scsi_inquiry.c 
 * as defined by hba_scsi_inquiry.h.  Because this library sits on 
 * top of the SNIA Common HBA Library, 
 * it requires hbaapi.dll to be in the $PATH.
 *
 * Supported Platforms  --
 *
 * Win32 circa NT, 2000,    compile under cl,  define MS_WIN32
 * Solaris circa 2.7,       compile under cc   define __UNIX__, __USE_POSIX__
 *
 * Language Version --  standard C
 * 
 * Original Author: Bill Pierce
 * Origination Date: 8-21-01
 *
 *****************************************************************************/
  
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef MS_WIN32
  #include <windows.h>
#else
  #define FALSE 0
  #define TRUE  1
#endif

#include "hbaapi.h"            /* The SNIA HBA API library. */
#include "hba_scsi_inquiry.h"  /* scsi inquiry function support. */
#include "hbaapiutils.h"       /* supports use of the hba_api. */
#include "portable_time.h"     /* cross-platform time functions. */
#include "getopt.h"            /* provides getopt functions. */

/*
 * Local data structures.
 */


/* HBA library versions compatible. */
#define MIN_COMPATIBLE_LIB_VERSION 1  
#define MAX_COMPATIBLE_LIB_VERSION 1

/* Allowed getopt options. */
#define ALLOWED_OPTIONS "vhc:i:l:q"
/* The default number of times an inquiry is tried. */
#define DEFAULT_INQUIRY_POLLS 3
/* The default target lun to ping. */
#define DEFAULT_TARGET_LUN 0
/* The default for the quiet option */
#define DEFAULT_QUIET_OPTION 0

/*
 * Local function prototypes. 
 */

int extract_target_wwn_from_args(int argc, 
                                 char* argv[], 
                                 HBA_WWN * port_wwn
                                );
int compatible_library_version(HBA_UINT32 hba_lib_version);
void print_usage_and_exit();
void print_version_and_exit();
void print_ping_statistics();
void react_to_command_line_options(int argc, char* argv[]);

/*
 * Command line flags. 
 */

int        total_ping_count_option;
int        seconds_between_pings_option;
HBA_UINT64 lun_to_ping_option;
int        quiet_option;

/*
 * Global data. 
 */

int  CurrentAdapterIndex = 0;
char CurrentAdapterName[256];
int  numberofadapters;
int  overall_success = TRUE;

/* Ping statistics */
int  total_pings_attempted;
long total_of_ping_waits_ms;
int  successful_pings, unsuccessful_pings;

HBA_UINT32 Version;

/*
 *  Main routine. 
 */
int main(int argc, char* argv[])
{
	int inquiry_count;
	int inquiry_reply_status = FALSE;
	int argument_parsing_status = FALSE;
	SCSI_INQUIRY_RESPONSE inquiry_response;
	HBA_HANDLE adapter_handle;
	HBA_WWN target_port_wwn;

        /* Determine behavior based on command options. */
        react_to_command_line_options(argc, argv);

        // Load the library and verify at least one adapter. 
	if (HBA_LoadLibrary() == HBA_STATUS_ERROR)
	   {
	   fprintf(stderr, "Error loading HBA library hbaapi.dll.\n");
	   fprintf(stderr, "Check for correct install and config.\n");
           fprintf(stderr, "Exiting...\n");
	   exit (EXIT_FAILURE);
	}

       Version = HBA_GetVersion();
       if (!compatible_library_version(Version)) {
          fprintf(stderr, 
                  "HBA library version %d incompatible with fcping.\n",
                  Version
                 );
          fprintf(stderr,"A version between %d and %d is required.\n",
                  MIN_COMPATIBLE_LIB_VERSION,
                  MAX_COMPATIBLE_LIB_VERSION
                 );
          exit(EXIT_FAILURE);
       }

       // Make sure there is at least one adapter present.
       numberofadapters = HBA_GetNumberOfAdapters();
       if (numberofadapters < 1) {
          fprintf(stderr, "No HBA adapters found in host. Exiting...\n");
          exit(EXIT_FAILURE);
       }

       /* Get the name of the adapter */
       HBA_GetAdapterName(0, (char *)&CurrentAdapterName);

       /* Get a handle to the adapter and open it. */
       adapter_handle = HBA_OpenAdapter(&CurrentAdapterName[0]);

       /* Retrieve the WWN from the command line. */
       memset(&target_port_wwn, 0, sizeof(HBA_WWN));
       argument_parsing_status =
       extract_target_wwn_from_args(argc, argv, &target_port_wwn);
       if (!argument_parsing_status) {
          fprintf(stderr, "Unrecognized WorldwideName: %s\n", argv[argc-1]);
          print_usage_and_exit();
       }

       /* Attempt to send the SCSI inquiry DEFAULT_INQUIRY_POLLS times. */
       overall_success = TRUE;
       if (!quiet_option) {
          print_inquiry_going_out(target_port_wwn, lun_to_ping_option);
       }

       /* Initialize ping statistics. */
       total_of_ping_waits_ms   = 0;
       total_pings_attempted    = 0;
       successful_pings         = 0;
       unsuccessful_pings       = 0;
       
       for (inquiry_count = 0; 
	    inquiry_count < total_ping_count_option; 
	    inquiry_count++
	    ) {

           /* Time the inquiry request. */
	   TIME_NOW time_before, time_after;
           double time_difference;
	   time_before = portable_time(&time_before); 
	   inquiry_reply_status = do_synchronous_time_limited_inquiry(
		                     adapter_handle,
                                     target_port_wwn,	   
                                     lun_to_ping_option,
                                     1000 * seconds_between_pings_option,
			             &inquiry_response
                                                                     );
	   time_after = portable_time(&time_after); 
           time_difference = portable_difftime(time_after, time_before);

           total_pings_attempted++;
           total_of_ping_waits_ms += (long) (1000 * time_difference);

          /* If it took a shorter time than the wait limit, wait before
           * sending the next ping.
           */
           if ((int) (1000 * time_difference) <
               1000 * seconds_between_pings_option 
              ) {
              portable_sleep_ms(1000 * seconds_between_pings_option -
                    (int) (1000 * time_difference)
                   );
           }
  
           /* See what we got back. */
           switch (inquiry_reply_status) {
               case SCSI_INQUIRY_REPLY_SUCCESS : {
                  overall_success &= TRUE;
                  successful_pings++;
                  if (!quiet_option) {
	             print_ping_reply_received(
                                          target_port_wwn,
                                          &inquiry_response,
                                          time_difference 
                                              );
                  }
                  break;
               }

               case SCSI_INQUIRY_REPLY_TIMED_OUT : {
                  overall_success &= FALSE;
                  unsuccessful_pings++;
                  if (!quiet_option) {
                     print_ping_request_timed_out();
                  }
                  break;
               }

               case SCSI_INQUIRY_REPLY_EMPTY : {
                  overall_success &= FALSE;
                  unsuccessful_pings++;
                  if (!quiet_option) {
                     print_ping_request_reply_empty();
                  }
                  break;
               }

               default : {
                  /* Assume there was an error. */
                  overall_success &= FALSE;
                  unsuccessful_pings++;
                  if (!quiet_option) {
                     print_ping_request_unsuccessful();
                  }
               }
           } /* switch. */
       } /* for. */

       /* Close the adapter handle. */
       HBA_CloseAdapter(adapter_handle);
      
       /* Output ping statistics. */
       if (!quiet_option) {
          print_ping_statistics();
       }

       /* Return. */
       if (overall_success) {
          return EXIT_SUCCESS;
       } else {
          return EXIT_FAILURE;
       }

} /* End main. */

/* Given the command line arguments, extract the wwn of target 
 * port that is to be pinged. Return TRUE on success, FALSE on failure.
 */
int extract_target_wwn_from_args(int argc, 
                                 char* argv[], 
                                 HBA_WWN * p_port_wwn) {

    /* The port wwn should be the last argument. */
    /* Was the WWN correctly formatted? */
    if (TRUE != StrToWWN(argv[argc-1], p_port_wwn)) {
       return FALSE;
    }
    return TRUE;
}

/* 
 * Determine if the specified HBA_API library version is compatible
 * with this program. Return 1 if compatible, 0 if not.
 */
int compatible_library_version(HBA_UINT32 hba_lib_version) {
   if ((MIN_COMPATIBLE_LIB_VERSION <= hba_lib_version) &&
       (MAX_COMPATIBLE_LIB_VERSION >= hba_lib_version) 
      ) {
        return 1;
   } else {
        return 0;
   }       
}

/* getopt defines this externally. */
extern char * optarg;

/* 
 * Go through the list of command line options and tailor 
 * behavior accordingly.
 */
void react_to_command_line_options(int argc, char* argv[]) {

   int opt = 0;
   int integer_option = 0;

   /* Set everything to default */
   total_ping_count_option = DEFAULT_INQUIRY_POLLS;
   lun_to_ping_option      = DEFAULT_TARGET_LUN;
   quiet_option            = DEFAULT_QUIET_OPTION; 
   seconds_between_pings_option = (int)
   DEFAULT_INQUIRY_SYNCHRONOUS_WAIT_LIMIT_MS / 1000; 

   /* Now process the command line. */
   while ((opt = getopt(argc, argv, ALLOWED_OPTIONS)) != EOF) {
  
      switch (opt) {
        case 'v' : {
           print_version_and_exit();
           break;
        }
        case 'h' : {
           print_usage_and_exit();
           break;
        }
        case 'c' : {
           if ((1 == sscanf(optarg, "%d", &integer_option)) && 
               (integer_option >= 0) 
              ) {
             total_ping_count_option = integer_option; 
           } else {
             fprintf(stderr, "Illegal value passed to -c: %s\n", optarg);
             print_usage_and_exit();
           }
           break;
        }
        case 'i' : {
           if ((1 == sscanf(optarg, "%d", &integer_option)) && 
               (integer_option >= 1) 
              ) {
             seconds_between_pings_option = integer_option; 
           } else {
             fprintf(stderr, "Illegal value passed to -i: %s\n", optarg);
             print_usage_and_exit();
           }
           break;
        }
        case 'l' : {
           if ((1 == sscanf(optarg, "%d", &integer_option)) && 
               (integer_option >= 0) 
              ) {
              IntToLun64(integer_option, &lun_to_ping_option); 
           } else {
             fprintf(stderr, "Illegal value passed to -l: %s\n", optarg);
             print_usage_and_exit();
           }
           break;
        }
        case 'q' : {
           quiet_option = 1;
           break;
        }
        default : {
           fprintf(stderr, "Unrecognized option: %c\n", opt);
           print_usage_and_exit();
           break;
        }
      } /* End switch */
   } /* End while. */
   
} /* End of react_to_command_line_options. */

/* Tally the ping statistics for this run and write them 
 * to stdout.
 */
void print_ping_statistics() {
    double average_ping_time;
    printf("\n%d successful and %d unsuccessful pings.\n",
           successful_pings, unsuccessful_pings
          );
    average_ping_time = ((double) total_of_ping_waits_ms / 
                         (double) total_pings_attempted) / 1000.0;
    printf("Average ping time: %2.3f s.\n", average_ping_time);
}

/* Print the command line usage of this program to stdout 
 * and exit indicating failure.
 */
void print_usage_and_exit() {
    printf("Usage: fcping [OPTION]... WORLD_WIDE_PORT_NAME\n");
    printf("WORLD_WIDE_PORT_NAME in upper case hex XX:XX:XX:XX:XX:XX:XX:XX\n");
    printf("\nOptions:\n");
    printf(" -v           print the version and exit\n");
    printf(" -h           print usage and exit\n");
    printf(" -c count     number of pings before exiting\n");
    printf(" -i interval  interval between pings in seconds\n");
    printf(" -l lun       FCP LUN to ping in decimal (default is LUN 0)\n");
    printf(" -q           quiet operation, no output to stdout\n");
    printf("\n");
    exit(EXIT_FAILURE);
}

/* Print version information to stdout and exit. */
void print_version_and_exit() {
    printf("Fibre Channel Ping utility, Version 1.0\n");
    printf("Copyright (C) Bill Pierce, 2002\n");
    printf("Dedicated to the memory of Mike Muuss, author of ping.\n\n");
    printf("This program is distributed in the hope that it will be useful,\n");
    printf("but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
    printf("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\n"); 
    exit(EXIT_SUCCESS);
}

/*****************************************************************************
 *
 *   $Log: $
 *
 *****************************************************************************/
